/*
This file is part of Silex - see http://projects.silexlabs.org/?/silex

Silex is © 2010-2011 Silex Labs and is released under the GPL License:

This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License (GPL) as published by the Free Software Foundation; either version 2 of the License, or (at your option) any later version. 

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

To read the license please visit http://www.gnu.org/copyleft/gpl.html
*/

package test.src.domElement;

/**
 * Units tests for DOMElements
 * @author Yannick DOMINGUEZ & Raphael HARMEL
 * 
 * 2011-08-10 PHP part is not complete yet.
 * 
 */

import haxe.Log;
import cocktail.domElement.ContainerDOMElement;
import cocktail.geom.GeomData;
import cocktail.domElement.DOMElement;
import cocktailCore.domElement.abstract.AbstractDOMElement;

import cocktail.domElement.GraphicDOMElement;
import cocktail.domElement.DOMElementData;
import utest.Assert;
import utest.Runner;
import utest.ui.Report;

import cocktail.resource.ResourceLoaderManager;
import cocktail.nativeElement.NativeElementData;
import cocktail.nativeElement.NativeElementManager;

#if (flash9 || cpp || nme)
import flash.display.Sprite;
import flash.Lib;

#elseif js
import js.Dom;
import js.Lib;
#end

class DOMElementTests 
{
	
	private static var rootDOMElement:ContainerDOMElement;
	
	public static function main()
	{
		rootDOMElement = new ContainerDOMElement(NativeElementManager.getRoot());
		
		var runner = new Runner();
		runner.addCase(new DOMElementTests());
		Report.create(runner);
		runner.run();
	}
	
	public function new() 
	{
	}
	
	#if flash9
	
	/**
	 * test adding/removing DOMElement,
	 * and getting/setting there properties for Flash as3
	 */
	public function testAddRemoveChild()
	{
		/**
		var spriteParentDOMElement:Sprite = new Sprite();
		spriteParentDOMElement.graphics.beginFill(0x00FF00, 1);
		spriteParentDOMElement.graphics.drawRect(0, 0, 200, 200);
		spriteParentDOMElement.graphics.endFill();
		//create a new DOMElement
		var parentDOMElement:ContainerDOMElement = new ContainerDOMElement(spriteParentDOMElement);
		
		//add the native Sprite to the Flash Stage
		Lib.current.addChild(parentDOMElement.nativeElement);
		
		var spriteChildDOMElement:Sprite = new Sprite();
		spriteChildDOMElement.graphics.beginFill(0xFF0000, 1);
		spriteChildDOMElement.graphics.drawRect(0, 0, 100, 100);
		spriteChildDOMElement.graphics.endFill();
		//create a child DOMElement
		var childDOMElement:DOMElement = new DOMElement(spriteChildDOMElement);
		childDOMElement.setWidth(100);
		childDOMElement.setHeight(100);
		
		//add the child DOMElement to the parent DOMElement
		parentDOMElement.addChild(childDOMElement);
		
		//compare the names to verify that the children's parent is the parent DOMElement
		Assert.same(childDOMElement.getParent().nativeElement.name, parentDOMElement.nativeElement.name);
		
		//test getting each of the properties of the child DOMElement
		Assert.equals(childDOMElement.getWidth(), 100);
		Assert.equals(childDOMElement.getHeight(), 100);
		Assert.equals(childDOMElement.getX(), 0);
		Assert.equals(childDOMElement.getY(), 0);
		Assert.equals(childDOMElement.getZIndex(), 0);
		
		//set the child height of the DOMElement and test the returned height
		childDOMElement.setHeight(200);
		Assert.equals(childDOMElement.getHeight(), 200);
		
		//remove the child and parent DOMElements from the display list
		parentDOMElement.removeChild(childDOMElement);
		Lib.current.removeChild(parentDOMElement.nativeElement);
		
		//check that the parent is now null
		Assert.same(childDOMElement.getParent(), null);*/
		
	}
	
	#elseif js
	
	/**
	 * test adding/removing DOMElement,
	 * and getting/setting there properties for JavaScript
	 */
	public function testAddRemoveChild()
	{
		/**
		//create a new div
		var divParentDOMElement:HtmlDom = js.Lib.document.createElement("div");
		divParentDOMElement.setAttribute("id","parentDiv");
		divParentDOMElement.innerHTML = "parent div";
		
		//create a new DOM object
		var parentDOMElement:ContainerDOMElement = new ContainerDOMElement(divParentDOMElement);
		
		//attach the div to the document body
		rootDOMElement.addChild(parentDOMElement);
		

		
		//create a new child div
		var divChildDOMElement:HtmlDom = js.Lib.document.createElement("div");
		divChildDOMElement.setAttribute("id", "childDiv");
		divChildDOMElement.innerHTML = "child div";
		
		//create a new child DOMElement
		var childDOMElement:DOMElement = new DOMElement(divChildDOMElement);
		
		childDOMElement.setWidth(300);
		childDOMElement.setHeight(200);
		childDOMElement.setX(100);
		childDOMElement.setY(50);
		
		//attach the child DOMElement on the parent DOMElement
		parentDOMElement.addChild(childDOMElement);
		
		//test getting each of the properties of the child DOMElement
		Assert.equals(childDOMElement.getWidth(), 300);
		Assert.equals(childDOMElement.getHeight(), 200);
		Assert.equals(childDOMElement.getX(), 100);
		Assert.equals(childDOMElement.getY(), 50);
		Assert.equals(childDOMElement.getZIndex(), 0);
		
		
		//check the parent of the child DOMElement
		//Assert.same(childDOMElement.getParent(), parentDOMElement);
		
		//remove the child DOMElement from the div
		parentDOMElement.removeChild(childDOMElement);
		
		//check that the child object has no more parent
		Assert.same(childDOMElement.getParent(), null);
		*/
	}

	#elseif php
	
	/**
	 * test adding/removing DOMElement,
	 * and getting/setting there properties for JavaScript
	 */
	public function testAddRemoveChild()
	{
		/**
		//create a new div
		var divParentDOMElement:Xml = Xml.createElement("div");
		divParentDOMElement.set("id","parentDiv");
		divParentDOMElement.addChild(Xml.createPCData("parent div"));
		
		//create a new DOM object
		var parentDOMElement:ContainerDOMElement = new ContainerDOMElement(divParentDOMElement);
		
		//attach the div to the document body
		rootDOMElement.addChild(parentDOMElement);

		//create a new child div
		var divChildDOMElement:Xml = Xml.createElement("div");
		divChildDOMElement.set("id","childDiv");
		divChildDOMElement.addChild(Xml.createPCData("child div"));
		
		//create a new child DOMElement
		var childDOMElement:DOMElement = new DOMElement(divChildDOMElement);
		
		childDOMElement.setWidth(300);
		childDOMElement.setHeight(200);
		childDOMElement.setX(100);
		childDOMElement.setY(50);
		
		//attach the child DOMElement on the parent DOMElement
		parentDOMElement.addChild(childDOMElement);
		
		//test getting each of the properties of the child DOMElement
		Assert.equals(childDOMElement.getWidth(), 300);
		Assert.equals(childDOMElement.getHeight(), 200);
		Assert.equals(childDOMElement.getX(), 100);
		Assert.equals(childDOMElement.getY(), 50);
		Assert.equals(childDOMElement.getZIndex(), 0);
		
		//check the parent of the child DOMElement
		//Assert.same(childDOMElement.getParent(), parentDOMElement);
		
		//remove the child DOMElement from the div
		parentDOMElement.removeChild(childDOMElement);
		
		//check that the child object has no more parent
		Assert.same(childDOMElement.getParent(), null);*/
	}
	#end
	
	/**
	 * Test visibility and opacity methods
	 */
	public function testVisibilityOpacity()
	{
		#if flash9
		var nativeDOMElement:Dynamic = new flash.display.Sprite();
		#elseif js
		var nativeDOMElement:Dynamic = js.Lib.document.createElement("canvas");
		#end
		
		//test with a graphic dom element to easily have a background
		var graphicDOMElement:GraphicDOMElement = new GraphicDOMElement(nativeDOMElement);
		
		graphicDOMElement.width = 200;
		graphicDOMElement.height = 200;
		graphicDOMElement.x = 0;
		graphicDOMElement.y = 100;
		
		var colorStop:ColorStopData = { color:Std.parseInt("0xFF0000") , alpha:100 };
		var fillStyle:FillStyleValue = monochrome(colorStop);
		
		var lineStyle:LineStyleValue = LineStyleValue.none;
		
		graphicDOMElement.beginFill(fillStyle, lineStyle);
		
		var cornerRadiuses:CornerRadiusData = {
			tlCornerRadius:10,
			trCornerRadius:0,
			blCornerRadius:20,
			brCornerRadius:0
		}
		
		
		graphicDOMElement.drawRect(0, 0, 100, 100, cornerRadiuses);
		graphicDOMElement.endFill();
		
		rootDOMElement.addChild(graphicDOMElement);
		
		graphicDOMElement.isVisible = false;
		Assert.equals(graphicDOMElement.isVisible, false);
		
		graphicDOMElement.isVisible = true ;
		Assert.equals(graphicDOMElement.isVisible, true);
		
		graphicDOMElement.alpha = 0.5;
		Assert.equals(graphicDOMElement.alpha, 0.5);
	}
	
	/**
	 * Test the manipulation of DOMElements z-indexes in Flash and JavaScript
	 */
	public function testZIndex()
	{
		//in Flash
		#if flash9
		//create a new DOMElement
		var spriteParentDOMElement:Sprite = new Sprite();
		spriteParentDOMElement.graphics.beginFill(0x00FF00, 0);
		spriteParentDOMElement.graphics.drawRect(0, 0, 200, 200);
		spriteParentDOMElement.graphics.endFill();
		var parentDOMElement:ContainerDOMElement = new ContainerDOMElement(spriteParentDOMElement);
		
		//add the native Sprite to the Flash Stage
		rootDOMElement.addChild(parentDOMElement);
		
		var spriteChildDOMElement1:Sprite = new Sprite();
		spriteChildDOMElement1.graphics.beginFill(0xFF0000, 1);
		spriteChildDOMElement1.graphics.drawRect(0, 0, 100, 100);
		spriteChildDOMElement1.graphics.endFill();
		//create a child DOMElement
		var childDOMElement1:ContainerDOMElement = new ContainerDOMElement(spriteChildDOMElement1);
		
		var spriteChildDOMElement2:Sprite = new Sprite();
		spriteChildDOMElement2.graphics.beginFill(0x00FF00, 1);
		spriteChildDOMElement2.graphics.drawRect(0, 0, 100, 100);
		spriteChildDOMElement2.graphics.endFill();
		//create a child DOMElement
		var childDOMElement2:ContainerDOMElement = new ContainerDOMElement(spriteChildDOMElement2);
		
		var spriteChildDOMElement3:Sprite = new Sprite();
		spriteChildDOMElement3.graphics.beginFill(0x0000FF, 1);
		spriteChildDOMElement3.graphics.drawRect(0, 0, 100, 100);
		spriteChildDOMElement3.graphics.endFill();
		//create a child DOMElement
		var childDOMElement3:ContainerDOMElement = new ContainerDOMElement(spriteChildDOMElement3);
		
		//in JavaScript
		#elseif js
		//create a new div
		var divParentDOMElement:HtmlDom = js.Lib.document.createElement("div");
		divParentDOMElement.setAttribute("id", "parentDiv");
		
		//create a new DOM object
		var parentDOMElement:ContainerDOMElement = new ContainerDOMElement(divParentDOMElement);
		
		//parentDOMElement.setWidth(100);
		//parentDOMElement.setHeight(100);
		
		//attach the div to the document body
		rootDOMElement.addChild(parentDOMElement);
		
		//create a new div
		var divChildDOMElement1:HtmlDom = js.Lib.document.createElement("div");
		divChildDOMElement1.setAttribute("id", "childDiv1");
		divChildDOMElement1.style.backgroundColor = "#FF0000";
		
		//create a new DOM object
		var childDOMElement1:ContainerDOMElement = new ContainerDOMElement(divChildDOMElement1);
		
		//create a new div
		var divChildDOMElement2:HtmlDom = js.Lib.document.createElement("div");
		divChildDOMElement2.setAttribute("id", "childDiv2");
		divChildDOMElement2.style.backgroundColor = "#00FF00";
		
		//create a new DOM object
		var childDOMElement2:ContainerDOMElement = new ContainerDOMElement(divChildDOMElement2);
		
		//create a new div
		var divChildDOMElement3:HtmlDom = js.Lib.document.createElement("div");
		divChildDOMElement3.setAttribute("id", "childDiv3");
		divChildDOMElement3.style.backgroundColor = "#0000FF";
		
		//create a new DOM object
		var childDOMElement3:ContainerDOMElement = new ContainerDOMElement(divChildDOMElement3);
		
		//in PHP
		#elseif php
		//create a new div
		var divParentDOMElement:Xml = Xml.createElement("div");
		divParentDOMElement.set("id", "parentDiv");
		
		//create a new DOM object
		var parentDOMElement:ContainerDOMElement = new ContainerDOMElement(divParentDOMElement);
		
		//parentDOMElement.setWidth(100);
		//parentDOMElement.setHeight(100);
		
		//attach the div to the document body
		rootDOMElement.addChild(parentDOMElement);
		
		//create a new div
		var divChildDOMElement1:Xml = Xml.createElement("div");
		divChildDOMElement1.set("id", "childDiv1");
		//divChildDOMElement1.style.backgroundColor = "#FF0000";
		
		//create a new DOM object
		var childDOMElement1:ContainerDOMElement = new ContainerDOMElement(divChildDOMElement1);
		
		//create a new div
		var divChildDOMElement2:Xml = Xml.createElement("div");
		divChildDOMElement2.set("id", "childDiv2");
		//divChildDOMElement2.style.backgroundColor = "#00FF00";
		
		//create a new DOM object
		var childDOMElement2:ContainerDOMElement = new ContainerDOMElement(divChildDOMElement2);
		
		//create a new div
		var divChildDOMElement3:Xml = Xml.createElement("div");
		divChildDOMElement3.set("id", "childDiv3");
		//divChildDOMElement3.style.backgroundColor = "#0000FF";
		
		//create a new DOM object
		var childDOMElement3:ContainerDOMElement = new ContainerDOMElement(divChildDOMElement3);
		
		#end
		
		//set the width/height and position of the DOMElement
		childDOMElement1.width = 100;
		childDOMElement1.height = 100;
		childDOMElement2.width  = 100;
		childDOMElement2.height = 100;
		childDOMElement3.width = 100;
		childDOMElement3.height = 100;
		
		childDOMElement2.x = 50;
		childDOMElement3.x = 80;
		
		//then add them to the parent node
		parentDOMElement.addChild(childDOMElement1);
		parentDOMElement.addChild(childDOMElement2);
		parentDOMElement.addChild(childDOMElement3);
		
		//check the default z-order set when the domElement is tested
		Assert.equals(1, childDOMElement2.zIndex);
		
		//check z-order manipulation and coherence with another
		//domElement z-index
		childDOMElement2.zIndex = 0;
		Assert.equals(1, childDOMElement1.zIndex);
		
		//check out of range z-index
		childDOMElement1.zIndex = 999;
		Assert.equals(2, childDOMElement1.zIndex);
		
	}
	
	public function testGlobalCoord()
	{
		var graph1:GraphicDOMElement = new GraphicDOMElement();
		graph1.width = 100;
		graph1.height = 100;
		graph1.x = 0;
		graph1.y = 0;
		graph1.beginFill(FillStyleValue.monochrome( { color:0xFF0000, alpha:100 } ), LineStyleValue.none);
		
		graph1.drawRect(0, 0, 100, 100);
		graph1.endFill();
		
		var graph2:GraphicDOMElement = new GraphicDOMElement();
		graph2.width = 100;
		graph2.height = 100;
		graph2.x = 0;
		graph2.y = 0;
		graph2.beginFill(FillStyleValue.monochrome( { color:0xFF0000, alpha:100 } ), LineStyleValue.none);
		
		graph2.drawRect(0, 0, 100, 100);
		graph2.endFill();
		
		var container:ContainerDOMElement = new ContainerDOMElement(NativeElementManager.createNativeElement(neutral));
		container.width = 200;
		container.height = 200;
		
		container.addChild(graph1);
		container.addChild(graph2);
		
		rootDOMElement.addChild(container);
		
		container.x = 100;
		container.y = 100;
		
		graph1.globalX = 300;
		graph2.globalY = 100;

		
		Assert.same(graph1.x, 200);
		Assert.same(graph2.globalY, 100);
	}
	
	/**
	 * Test the container dom element specific method
	 */
	public function testContainerDOMElement()
	{
		#if flash9
		var containerDomObject:ContainerDOMElement = new ContainerDOMElement(new Sprite());
		#elseif js
		var containerDomObject:ContainerDOMElement = new ContainerDOMElement(js.Lib.document.createElement("div"));
		#elseif php
		var containerDomObject:ContainerDOMElement = new ContainerDOMElement(Xml.createElement('div'));
		#end
		
		rootDOMElement.addChild(containerDomObject);
		
		containerDomObject.semantic = "nav";
		
		Assert.same(containerDomObject.semantic, "nav");
	}
	
}